@@ -2,8 +2,8 @@
 
 SDIR=`dirname $0` 
 
-if [ -e $SDIR/setenv-build-jogl-x86_64.sh ] ; then
-    . $SDIR/setenv-build-jogl-x86_64.sh
+if [ -e $SDIR/setenv-build-jogamp-x86_64.sh ] ; then
+    . $SDIR/setenv-build-jogamp-x86_64.sh
 fi
 
 #    -Dc.compiler.debug=true 
@@ -20,9 +20,9 @@ fi
 #    -Dc.compiler.debug=true  \
 #    -Djavacdebuglevel="source,lines,vars" \
 
-export SOURCE_LEVEL=1.6
-export TARGET_LEVEL=1.6
-export TARGET_RT_JAR=/opt-share/jre1.6.0_30/lib/rt.jar
+export SOURCE_LEVEL=1.8
+export TARGET_LEVEL=1.8
+export TARGET_RT_JAR=/opt-share/jre1.8.0_212/lib/rt.jar
 
 export GLUEGEN_PROPERTIES_FILE="lib/gluegen-clang.properties"
 # or -Dgcc.compat.compiler=clang
