@@ -2,8 +2,11 @@
 
 SDIR=`dirname $0` 
 
-if [ -e $SDIR/setenv-build-jogl-x86_64.sh ] ; then
-    . $SDIR/setenv-build-jogl-x86_64.sh
+# export J2RE_HOME=/usr/lib/jvm/java-11-openjdk-amd64
+# export JAVA_HOME=/usr/lib/jvm/java-11-openjdk-amd64
+
+if [ -e $SDIR/setenv-build-jogamp-x86_64.sh ] ; then
+    . $SDIR/setenv-build-jogamp-x86_64.sh
 fi
 
 #    -Dc.compiler.debug=true 
@@ -13,16 +16,12 @@ fi
 #    -DisLinuxAMD64=true \
 #    -DisX11=true \
 #
-#    -Dtarget.sourcelevel=1.6 \
-#    -Dtarget.targetlevel=1.6 \
-#    -Dtarget.rt.jar=/opt-share/jre1.6.0_30/lib/rt.jar \
-#
 #    -Dc.compiler.debug=true  \
 #    -Djavacdebuglevel="source,lines,vars" \
 
-export SOURCE_LEVEL=1.6
-export TARGET_LEVEL=1.6
-export TARGET_RT_JAR=/opt-share/jre1.6.0_30/lib/rt.jar
+export SOURCE_LEVEL=1.8
+export TARGET_LEVEL=1.8
+export TARGET_RT_JAR=/opt-share/jre1.8.0_212/lib/rt.jar
 
 #export JOGAMP_JAR_CODEBASE="Codebase: *.jogamp.org"
 export JOGAMP_JAR_CODEBASE="Codebase: *.goethel.localnet"
