@@ -33,17 +33,15 @@ import java.io.OutputStream;
 import java.io.PrintStream;
 import java.io.RandomAccessFile;
 import java.lang.ref.WeakReference;
-import java.lang.reflect.Method;
 import java.nio.ByteBuffer;
 import java.nio.MappedByteBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.channels.FileChannel.MapMode;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
+
+import com.jogamp.common.os.Platform.OSType;
 
 import jogamp.common.Debug;
-
-import com.jogamp.common.os.Platform;
+import jogamp.common.os.PlatformPropsImpl;
 
 /**
  * An {@link InputStream} implementation based on an underlying {@link FileChannel}'s memory mapped {@link ByteBuffer},
@@ -139,8 +137,7 @@ public class MappedByteBufferInputStream extends InputStream {
     static final boolean DEBUG;
 
     static {
-        Platform.initSingleton();
-        if( Platform.is32Bit() ) {
+        if( PlatformPropsImpl.CPU_ARCH.is32Bit ) {
             DEFAULT_SLICE_SHIFT = 29;
         } else {
             DEFAULT_SLICE_SHIFT = 30;
@@ -163,10 +160,6 @@ public class MappedByteBufferInputStream extends InputStream {
 
     private int refCount;
 
-    private Method mbbCleaner;
-    private Method cClean;
-    private boolean cleanerInit;
-    private boolean hasCleaner;
     private CacheMode cmode;
 
     private int sliceIdx;
@@ -191,10 +184,12 @@ public class MappedByteBufferInputStream extends InputStream {
             }
         }
         long fcSz = 0, pos = 0, rem = 0;
-        try {
-            fcSz = fc.size();
-        } catch (final IOException e) {
-            e.printStackTrace();
+        if( fc.isOpen() ) {
+            try {
+                fcSz = fc.size();
+            } catch (final IOException e) {
+                e.printStackTrace();
+            }
         }
         if( 0 < refCount ) {
             try {
@@ -229,14 +224,16 @@ public class MappedByteBufferInputStream extends InputStream {
         notifyLengthChange( totalSize );
 
         this.refCount = 1;
-        this.cleanerInit = false;
-        this.hasCleaner = false;
         this.cmode = cmode;
 
         this.sliceIdx = currSliceIdx;
         this.mark = -1;
 
         currentSlice().position(0);
+
+        if( MappedByteBufferInputStream.DEBUG ) {
+            this.dbgDump("CTOR", System.err);
+        }
     }
 
     /**
@@ -330,6 +327,9 @@ public class MappedByteBufferInputStream extends InputStream {
                 }
             }
         }
+        if( MappedByteBufferInputStream.DEBUG ) {
+            this.dbgDump("Close", System.err);
+        }
     }
 
     final FileChannel.MapMode getMapMode() { return mmode; }
@@ -366,7 +366,7 @@ public class MappedByteBufferInputStream extends InputStream {
             currentPosition = -1L;
         }
         if( fc.size() != newTotalSize ) {
-            if( Platform.OSType.WINDOWS == Platform.getOSType() ) {
+            if( OSType.WINDOWS == PlatformPropsImpl.OS_TYPE ) {
                 // On Windows, we have to close all mapped slices.
                 // Otherwise we will receive:
                 // java.io.IOException: The requested operation cannot be performed on a file with a user-mapped section open
@@ -441,10 +441,9 @@ public class MappedByteBufferInputStream extends InputStream {
             }
             position2( Math.min(prePosition, newTotalSize) ); // -> clipped position (set currSlice and re-map/-pos buffer)
         }
-        /* if( DEBUG ) {
-            System.err.println("notifyLengthChange.X: "+slices[currSlice]);
-            dbgDump("notifyLengthChange.X:", System.err);
-        } */
+        if( MappedByteBufferInputStream.DEBUG ) {
+            this.dbgDump("NotifyLengthChange", System.err);
+        }
     }
 
     /**
@@ -551,6 +550,21 @@ public class MappedByteBufferInputStream extends InputStream {
         }
     }
 
+    /**
+     * Releases the mapped {@link ByteBuffer} slices.
+     * @throws IOException if a buffer slice operation failed.
+     */
+    public final synchronized void flushSlices() throws IOException {
+        if( null != slices ) {
+            for(int i=0; i<sliceCount; i++) {
+                flushSlice(i, synchronous);
+            }
+        }
+        if( MappedByteBufferInputStream.DEBUG ) {
+            this.dbgDump("FlushSlices", System.err);
+        }
+    }
+
     synchronized void syncSlice(final ByteBuffer s) throws IOException {
         syncSlice(s, synchronous);
     }
@@ -630,58 +644,16 @@ public class MappedByteBufferInputStream extends InputStream {
         }
     }
     private synchronized boolean cleanBuffer(final ByteBuffer mbb, final boolean syncBuffer) throws IOException {
-        if( !cleanerInit ) {
-            initCleaner(mbb);
-        }
         syncSlice(mbb, syncBuffer);
         if( !mbb.isDirect() ) {
             return false;
         }
-        boolean res = false;
-        if ( hasCleaner ) {
-            try {
-                cClean.invoke(mbbCleaner.invoke(mbb));
-                res = true;
-            } catch(final Throwable t) {
-                hasCleaner = false;
-                if( DEBUG ) {
-                    System.err.println("Caught "+t.getMessage());
-                    t.printStackTrace();
-                }
-            }
-        }
-        if( !res && CacheMode.FLUSH_PRE_HARD == cmode ) {
+        if( !Buffers.Cleaner.clean(mbb) && CacheMode.FLUSH_PRE_HARD == cmode ) {
             cmode = CacheMode.FLUSH_PRE_SOFT;
+            return false;
+        } else {
+            return true;
         }
-        return res;
-    }
-    private synchronized void initCleaner(final ByteBuffer bb) {
-        final Method[] _mbbCleaner = { null };
-        final Method[] _cClean = { null };
-        AccessController.doPrivileged(new PrivilegedAction<Object>() {
-            @Override
-            public Object run() {
-                try {
-                    _mbbCleaner[0] = bb.getClass().getMethod("cleaner");
-                    _mbbCleaner[0].setAccessible(true);
-                    _cClean[0] = Class.forName("sun.misc.Cleaner").getMethod("clean");
-                    _cClean[0].setAccessible(true);
-                } catch(final Throwable t) {
-                    if( DEBUG ) {
-                        System.err.println("Caught "+t.getMessage());
-                        t.printStackTrace();
-                    }
-                }
-                return null;
-            } } );
-        mbbCleaner = _mbbCleaner[0];
-        cClean = _cClean[0];
-        final boolean res = null != mbbCleaner && null != cClean;
-        if( DEBUG ) {
-            System.err.println("initCleaner: Has cleaner: "+res+", mbbCleaner "+mbbCleaner+", cClean "+cClean);
-        }
-        hasCleaner = res;
-        cleanerInit = true;
     }
 
     /**
