@@ -58,12 +58,14 @@ public class MachineDataInfo {
   /*                              arch   os          int, long, float, doubl, ldoubl,  ptr,   page */
   private final static int[] size_arm_mips_32     =  { 4,    4,     4,     8,      8,    4,   4096 };
   private final static int[] size_x86_32_unix     =  { 4,    4,     4,     8,     12,    4,   4096 };
+  private final static int[] size_x86_32_android  =  { 4,    4,     4,     8,      8,    4,   4096 };
   private final static int[] size_x86_32_macos    =  { 4,    4,     4,     8,     16,    4,   4096 };
   private final static int[] size_ppc_32_unix     =  { 4,    4,     4,     8,     16,    4,   4096 };
   private final static int[] size_sparc_32_sunos  =  { 4,    4,     4,     8,     16,    4,   8192 };
   private final static int[] size_x86_32_windows  =  { 4,    4,     4,     8,     12,    4,   4096 };
   private final static int[] size_lp64_unix       =  { 4,    8,     4,     8,     16,    8,   4096 };
   private final static int[] size_x86_64_windows  =  { 4,    4,     4,     8,     16,    8,   4096 };
+  private final static int[] size_arm64_ios       =  { 4,    8,     4,     8,      8,    8,   8192 };
 
   /*                               arch   os          i8, i16, i32, i64, int, long, float, doubl, ldoubl, ptr */
   private final static int[] align_arm_mips_32    =  { 1,   2,   4,   8,   4,    4,     4,     8,      8,   4 };
@@ -74,11 +76,12 @@ public class MachineDataInfo {
   private final static int[] align_x86_32_windows =  { 1,   2,   4,   8,   4,    4,     4,     8,      4,   4 };
   private final static int[] align_lp64_unix      =  { 1,   2,   4,   8,   4,    8,     4,     8,     16,   8 };
   private final static int[] align_x86_64_windows =  { 1,   2,   4,   8,   4,    4,     4,     8,     16,   8 };
+  private final static int[] align_arm64_ios      =  { 1,   2,   4,   8,   4,    8,     4,     8,      8,   8 };
 
   /**
    * Static enumeration of {@link MachineDataInfo} instances
    * used for high performance data size and alignment lookups,
-   * e.g. for generated structures.
+   * e.g. for generated structures using the {@link MachineDataInfo.StaticConfig} index.
    * <p>
    * The value {@link MachineDataInfo#pageSizeInBytes} shall be ignored
    * for static instances!
@@ -98,6 +101,8 @@ public class MachineDataInfo {
       ARM_MIPS_32(     size_arm_mips_32,   align_arm_mips_32),
       /** {@link Platform.CPUType#X86_32} Unix */
       X86_32_UNIX(    size_x86_32_unix,    align_x86_32_unix),
+      /** {@link Platform.CPUType#X86_32} Android/Bionic */
+      X86_32_ANDROID( size_x86_32_android, align_x86_32_unix),
       /** {@link Platform.CPUType#X86_32} MacOS (Special case gcc4/OSX) */
       X86_32_MACOS(   size_x86_32_macos,   align_x86_32_macos),
       /** {@link Platform.CPUType#PPC} Unix */
@@ -109,8 +114,10 @@ public class MachineDataInfo {
       /** LP64 Unix, e.g.: {@link Platform.CPUType#X86_64} Unix, {@link Platform.CPUType#ARM64} EABI, {@link Platform.CPUType#PPC64} Unix, .. */
       LP64_UNIX(      size_lp64_unix,    align_lp64_unix),
       /** {@link Platform.CPUType#X86_64} Windows */
-      X86_64_WINDOWS( size_x86_64_windows, align_x86_64_windows);
-      // 8
+      X86_64_WINDOWS( size_x86_64_windows, align_x86_64_windows),
+      /** {@link Platform.CPUType#ARM64 } iOS */
+      ARM64_IOS( size_arm64_ios, align_arm64_ios);
+      // 9
 
       public final MachineDataInfo md;
 
