@@ -1,5 +1,5 @@
 /**
- * Copyright 2010 JogAmp Community. All rights reserved.
+ * Copyright 2010-2023 JogAmp Community. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are
  * permitted provided that the following conditions are met:
@@ -521,26 +521,12 @@ public class Platform extends PlatformPropsImpl {
     //
 
     /**
-     * Returns the unix based current time in milliseconds, based on <code>gettimeofday(..)</code>.
-     * <p>
-     * This is an alternative to {@link System#currentTimeMillis()} and {@link System#nanoTime()}.
-     * While the named {@link System} methods do provide the required precision,
-     * <code>gettimeofday()</code> <i>also</i> guarantees time accuracy, i.e. update interval.
-     * </p>
-     * @see #currentTimeMicros()
+     * Returns the unix based current time in milliseconds, see {@link Clock#currentTimeMillis()}.
+     * @see Clock#currentTimeMillis()
      */
-    public static native long currentTimeMillis();
-
-    /**
-     * Returns the unix based current time in microseconds, based on <code>gettimeofday(..)</code>.
-     * <p>
-     * This is an alternative to {@link System#currentTimeMillis()} and {@link System#nanoTime()}.
-     * While the named {@link System} methods do provide the required precision,
-     * <code>gettimeofday()</code> <i>also</i> guarantees time accuracy, i.e. update interval.
-     * </p>
-     * @see #currentTimeMillis()
-     */
-    public static native long currentTimeMicros();
+    public static long currentTimeMillis() {
+        return Clock.currentTimeMillis();
+    }
 
     /**
      * Returns the estimated sleep jitter value in nanoseconds.
@@ -554,11 +540,11 @@ public class Platform extends PlatformPropsImpl {
     }
     private static long getCurrentSleepJitterImpl(final long nsDuration, final int splitInLoops) {
         final long nsPeriod = nsDuration / splitInLoops;
-        final long t0_ns = System.nanoTime();
+        final long t0_ns = Clock.currentNanos();
         for(int i=splitInLoops; i>0; i--) {
             try { TimeUnit.NANOSECONDS.sleep(nsPeriod); } catch (final InterruptedException e) { }
         }
-        return  ( ( System.nanoTime() - t0_ns ) - nsDuration ) / splitInLoops;
+        return  ( ( Clock.currentNanos() - t0_ns ) - nsDuration ) / splitInLoops;
     }
 
 }
