@@ -1,4 +1,5 @@
-/*
+/**
+ * Copyright (c) 2010-2023 JogAmp Community. All rights reserved.
  * Copyright (c) 2003 Sun Microsystems, Inc. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,6 +41,7 @@
 package com.jogamp.gluegen;
 
 import com.jogamp.gluegen.cgram.types.FunctionSymbol;
+import com.jogamp.gluegen.cgram.types.FunctionType;
 import com.jogamp.gluegen.cgram.types.Type;
 
 import java.util.ArrayList;
@@ -68,7 +70,6 @@ public class MethodBinding {
   private boolean        signatureUsesCPrimitivePointers;
   private boolean        signatureUsesCArrays;
   private boolean        signatureUsesJavaPrimitiveArrays;
-  private boolean        signatureRequiresStaticInitialization;
   private int            thisPointerIndex = -1;
 
   /**
@@ -95,7 +96,6 @@ public class MethodBinding {
     this.signatureUsesCPrimitivePointers  = bindingToCopy.signatureUsesCPrimitivePointers;
     this.signatureUsesCArrays             = bindingToCopy.signatureUsesCArrays;
     this.signatureUsesJavaPrimitiveArrays = bindingToCopy.signatureUsesJavaPrimitiveArrays;
-    this.signatureRequiresStaticInitialization = bindingToCopy.signatureRequiresStaticInitialization;
     this.thisPointerIndex                 = bindingToCopy.thisPointerIndex;
   }
 
@@ -155,6 +155,176 @@ public class MethodBinding {
         return sym.getArgumentType(i);
     }
 
+    /**
+     * Returns the function parameter list, i.e. a comma separated list of argument type and name.
+     * @param buf StringBuilder instance
+     * @param useTypedef if true and type is typedef'ed, use its name
+     * @param callingConvention optional calling-convention
+     * @return given StringBuilder instance
+     */
+    public StringBuilder getCParameterList(final StringBuilder buf, final boolean useTypedef, final String callingConvention) {
+        return getCParameterList(buf, useTypedef, callingConvention, null);
+    }
+    /**
+     * Returns the function parameter list, i.e. a comma separated list of argument type and name.
+     * @param buf StringBuilder instance
+     * @param useTypedef if true and type is typedef'ed, use its name
+     * @param callingConvention optional calling-convention
+     * @param exclude optional list of excluded parameter indices
+     * @return given StringBuilder instance
+     */
+    public StringBuilder getCParameterList(final StringBuilder buf, final boolean useTypedef, final String callingConvention, final List<Integer> exclude) {
+        forEachParameter( ( final int idx, final int consumedCount, final Type cType, final JavaType jType, final String name ) -> {
+            if( !cType.isVoid() && ( null == exclude || !exclude.contains(idx) ) ) {
+                if( 0 < consumedCount ) {
+                    buf.append(", ");
+                }
+                if( useTypedef && cType.isTypedef() ) {
+                    buf.append( cType.getName() );
+                    if (name != null) {
+                        buf.append(" ");
+                        buf.append(name);
+                    }
+                } else if ( cType.isFunctionPointer() ) {
+                    final FunctionType ft = cType.getTargetFunction();
+                    buf.append( ft.toString(name, callingConvention, false, true) );
+                } else if (cType.isArray()) {
+                    buf.append( cType.asArray().toString(name) );
+                } else {
+                    buf.append( cType.getCName(true) );
+                    if (name != null) {
+                        buf.append(" ");
+                        buf.append(name);
+                    }
+                }
+                return true;
+            } else {
+                return false;
+            }
+        } );
+        return buf;
+    }
+
+    /**
+     * Returns the function parameter list, i.e. a comma separated list of argument type and name.
+     * @param buf StringBuilder instance
+     * @return given StringBuilder instance
+     */
+    public StringBuilder getJavaParameterList(final StringBuilder buf) {
+        return getJavaParameterList(buf, null);
+    }
+    /**
+     * Returns the function parameter list, i.e. a comma separated list of argument type and name.
+     * @param buf StringBuilder instance
+     * @param exclude optional list of excluded parameter indices
+     * @return given StringBuilder instance
+     */
+    public StringBuilder getJavaParameterList(final StringBuilder buf, final List<Integer> exclude) {
+        forEachParameter( ( final int idx, final int consumedCount, final Type cType, final JavaType jType, final String name ) -> {
+            if( !cType.isVoid() && ( null == exclude || !exclude.contains(idx) ) ) {
+                if( 0 < consumedCount ) {
+                    buf.append(", ");
+                }
+                buf.append(jType+" "+name);
+                return true;
+            } else {
+                return false;
+            }
+        } );
+        return buf;
+    }
+    /**
+     * Returns the function parameter list, i.e. a comma separated list of argument type and name.
+     * @param buf StringBuilder instance
+     * @param include list of explicit included parameter indices
+     * @param addTailSeparator add a comma separator in the end if result is not empty
+     * @return given StringBuilder instance
+     */
+    public StringBuilder getJavaSelectParameter(final StringBuilder buf, final List<Integer> include, final boolean addTailSeparator) {
+        forEachParameter( ( final int idx, final int consumedCount, final Type cType, final JavaType jType, final String name ) -> {
+            if( !cType.isVoid() && include.contains(idx) ) {
+                if( 0 < consumedCount ) {
+                    buf.append(", ");
+                }
+                buf.append(jType+" "+name);
+                return true;
+            } else {
+                return false;
+            }
+        } );
+        if( addTailSeparator && buf.length() > 0 ) {
+            buf.append(", ");
+        }
+        return buf;
+    }
+    public StringBuilder getJavaCallArgumentList(final StringBuilder buf, final List<Integer> exclude) {
+        forEachParameter( ( final int idx, final int consumedCount, final Type cType, final JavaType jType, final String name ) -> {
+            if( !cType.isVoid() && ( null == exclude || !exclude.contains(idx) ) ) {
+                if( 0 < consumedCount ) {
+                    buf.append(", ");
+                }
+                buf.append(name);
+                return true;
+            } else {
+                return false;
+            }
+        } );
+        return buf;
+    }
+    public StringBuilder getJavaCallSelectArguments(final StringBuilder buf, final List<Integer> include, final boolean addTailSeparator) {
+        forEachParameter( ( final int idx, final int consumedCount, final Type cType, final JavaType jType, final String name ) -> {
+            if( !cType.isVoid() && include.contains(idx) ) {
+                if( 0 < consumedCount ) {
+                    buf.append(", ");
+                }
+                buf.append(name);
+                return true;
+            } else {
+                return false;
+            }
+        } );
+        if( addTailSeparator && buf.length() > 0 ) {
+            buf.append(", ");
+        }
+        return buf;
+    }
+
+    /** {@link #forEachParameter(ParameterConsumer)} Consumer */
+    public static interface ParameterConsumer  {
+        /**
+         * Accept the arguments of the traversed collection element
+         * and return true if consumed. Consumed elements will increased passed `consumedCount` state.
+         * @param idx index of current element, ranges [0 .. size-1]
+         * @param consumedCount number of consumed elements, useful for e.g. `boolean needsSeparator =  0 < consumedCount`
+         * @param cType C Type of argument
+         * @param jType Java Type of argument
+         * @param name argument name
+         * @return true to signal consumed and have traversing loop increment `consumedCount`, otherwise false
+         */
+        boolean accept(int idx, int consumedCount, Type cType, JavaType jType, String name);
+    }
+    public int forEachParameter(final ParameterConsumer c) {
+        final int n = getNumArguments();
+        int consumedCount = 0;
+        for (int i = 0; i < n; i++) {
+            if( c.accept(i, consumedCount, getCArgumentType(i), getJavaArgumentType(i), getArgumentName(i)) ) {
+                ++consumedCount;
+            }
+        }
+        return consumedCount;
+    }
+
+    public final boolean isReturnCompoundByValue() {
+        final Type cReturnType = getCReturnType();
+        if (cReturnType.isVoid()) {
+            return false;
+        }
+        if (javaReturnType.isPrimitive()) {
+            return false;
+        }
+        return !cReturnType.isPointer() && javaReturnType.isCompoundTypeWrapper();
+    }
+
     /** Returns the {@link FunctionSymbol}. */
     public FunctionSymbol getCSymbol() {
         return sym;
@@ -271,21 +441,6 @@ public class MethodBinding {
     return signatureUsesCompoundTypeWrappers;
   }
 
-  /**
-   * Returns true if the wrapper implementation requires
-   * static native code to be initialized, see {@link JavaConfiguration#forceStaticInitCode(String)}.
-   * <p>
-   * Currently triggered by:
-   * <ul>
-   *   <li>Return type is a "compound type" and not a pointer</li>
-   * </ul>
-   * </p>
-   */
-  public boolean signatureRequiresStaticInitialization() {
-    computeSignatureProperties();
-    return signatureRequiresStaticInitialization;
-  }
-
   /**
    * Returns true if the return type or any of the outgoing arguments
    * in the method's signature use arrays of "compound type wrappers",
@@ -359,17 +514,11 @@ public class MethodBinding {
     signatureUsesCPrimitivePointers = false;
     signatureUsesCArrays = false;
     signatureUsesJavaPrimitiveArrays = false;
-    signatureRequiresStaticInitialization = false;
 
     if ( javaReturnType.isCompoundTypeWrapper() ) {
       // Needs wrapping and/or setting of byte order (neither of which
       // can be done easily from native code)
       signatureUsesCompoundTypeWrappers = true;
-
-      final Type cReturnType = getCReturnType();
-      if ( !cReturnType.isPointer() ) { // FIXME: Compound call-by-value
-          signatureRequiresStaticInitialization = true;
-      }
     }
 
     if (javaReturnType.isNIOBuffer() ||
@@ -384,7 +533,7 @@ public class MethodBinding {
     if (cRetType.isArray()) {
       // Needs checking of array lengths
       signatureUsesCArrays = true;
-      if (cRetType.asArray().getElementType().isPrimitive()) {
+      if (cRetType.asArray().getTargetType().isPrimitive()) {
         signatureUsesCPrimitivePointers = true;
       }
     }
@@ -426,7 +575,7 @@ public class MethodBinding {
       if (cArgType.isArray()) {
         // Needs checking of array lengths
         signatureUsesCArrays = true;
-        if (cArgType.asArray().getElementType().isPrimitive()) {
+        if (cArgType.asArray().getTargetType().isPrimitive()) {
           signatureUsesCPrimitivePointers = true;
         }
       }
